@@ -18,27 +18,27 @@ import types
 import warnings
 from collections.abc import Iterable
 from contextlib import redirect_stderr, redirect_stdout
-from typing import Any, Union
+from typing import TYPE_CHECKING, Any
 
 from astroid import bases, nodes
 from astroid.const import _EMPTY_OBJECT_MARKER, IS_PYPY
-from astroid.manager import AstroidManager
 from astroid.nodes import node_classes
 
+if TYPE_CHECKING:
+    from astroid.manager import AstroidManager
+
 logger = logging.getLogger(__name__)
 
 
-_FunctionTypes = Union[
-    types.FunctionType,
-    types.MethodType,
-    types.BuiltinFunctionType,
-    types.WrapperDescriptorType,
-    types.MethodDescriptorType,
-    types.ClassMethodDescriptorType,
-]
+_FunctionTypes = (
+    types.FunctionType
+    | types.MethodType
+    | types.BuiltinFunctionType
+    | types.WrapperDescriptorType
+    | types.MethodDescriptorType
+    | types.ClassMethodDescriptorType
+)
 
-# the keys of CONST_CLS eg python builtin types
-_CONSTANTS = tuple(node_classes.CONST_CLS)
 TYPE_NONE = type(None)
 TYPE_NOTIMPLEMENTED = type(NotImplemented)
 TYPE_ELLIPSIS = type(...)
@@ -49,24 +49,29 @@ def _attach_local_node(parent, node, name: str) -> None:
     parent.add_local_node(node)
 
 
-def _add_dunder_class(func, member) -> None:
+def _add_dunder_class(func, parent: nodes.NodeNG, member) -> None:
     """Add a __class__ member to the given func node, if we can determine it."""
     python_cls = member.__class__
     cls_name = getattr(python_cls, "__name__", None)
     if not cls_name:
         return
     cls_bases = [ancestor.__name__ for ancestor in python_cls.__bases__]
-    ast_klass = build_class(cls_name, cls_bases, python_cls.__doc__)
+    doc = python_cls.__doc__ if isinstance(python_cls.__doc__, str) else None
+    ast_klass = build_class(cls_name, parent, cls_bases, doc)
     func.instance_attrs["__class__"] = [ast_klass]
 
 
+def build_dummy(runtime_object) -> nodes.EmptyNode:
+    enode = nodes.EmptyNode()
+    enode.object = runtime_object
+    return enode
+
+
 def attach_dummy_node(node, name: str, runtime_object=_EMPTY_OBJECT_MARKER) -> None:
     """create a dummy node and register it in the locals of the given
     node with the specified name
     """
-    enode = nodes.EmptyNode()
-    enode.object = runtime_object
-    _attach_local_node(node, enode, name)
+    _attach_local_node(node, build_dummy(runtime_object), name)
 
 
 def attach_const_node(node, name: str, value) -> None:
@@ -96,7 +101,10 @@ def build_module(name: str, doc: str | None = None) -> nodes.Module:
 
 
 def build_class(
-    name: str, basenames: Iterable[str] = (), doc: str | None = None
+    name: str,
+    parent: nodes.NodeNG,
+    basenames: Iterable[str] = (),
+    doc: str | None = None,
 ) -> nodes.ClassDef:
     """Create and initialize an astroid ClassDef node."""
     node = nodes.ClassDef(
@@ -105,7 +113,7 @@ def build_class(
         col_offset=0,
         end_lineno=0,
         end_col_offset=0,
-        parent=nodes.Unknown(),
+        parent=parent,
     )
     node.postinit(
         bases=[
@@ -128,6 +136,7 @@ def build_class(
 
 def build_function(
     name: str,
+    parent: nodes.NodeNG,
     args: list[str] | None = None,
     posonlyargs: list[str] | None = None,
     defaults: list[Any] | None = None,
@@ -141,7 +150,7 @@ def build_function(
         name,
         lineno=0,
         col_offset=0,
-        parent=node_classes.Unknown(),
+        parent=parent,
         end_col_offset=0,
         end_lineno=0,
     )
@@ -258,11 +267,11 @@ def register_arguments(func: nodes.FunctionDef, args: list | None = None) -> Non
 
 
 def object_build_class(
-    node: nodes.Module | nodes.ClassDef, member: type, localname: str
+    node: nodes.Module | nodes.ClassDef, member: type
 ) -> nodes.ClassDef:
     """create astroid for a living class object"""
     basenames = [base.__name__ for base in member.__bases__]
-    return _base_class_object_build(node, member, basenames, localname=localname)
+    return _base_class_object_build(node, member, basenames)
 
 
 def _get_args_info_from_callable(
@@ -300,8 +309,8 @@ def _get_args_info_from_callable(
 
 
 def object_build_function(
-    node: nodes.Module | nodes.ClassDef, member: _FunctionTypes, localname: str
-) -> None:
+    node: nodes.Module | nodes.ClassDef, member: _FunctionTypes
+) -> nodes.FunctionDef:
     """create astroid for a living function object"""
     (
         args,
@@ -311,59 +320,49 @@ def object_build_function(
         kwonly_defaults,
     ) = _get_args_info_from_callable(member)
 
-    func = build_function(
-        getattr(member, "__name__", None) or localname,
+    return build_function(
+        getattr(member, "__name__", "<no-name>"),
+        node,
         args,
         posonlyargs,
         defaults,
-        member.__doc__,
+        member.__doc__ if isinstance(member.__doc__, str) else None,
         kwonlyargs=kwonlyargs,
         kwonlydefaults=kwonly_defaults,
     )
 
-    node.add_local_node(func, localname)
-
 
 def object_build_datadescriptor(
-    node: nodes.Module | nodes.ClassDef, member: type, name: str
+    node: nodes.Module | nodes.ClassDef, member: type
 ) -> nodes.ClassDef:
     """create astroid for a living data descriptor object"""
-    return _base_class_object_build(node, member, [], name)
+    return _base_class_object_build(node, member, [])
 
 
 def object_build_methoddescriptor(
     node: nodes.Module | nodes.ClassDef,
     member: _FunctionTypes,
-    localname: str,
-) -> None:
+) -> nodes.FunctionDef:
     """create astroid for a living method descriptor object"""
     # FIXME get arguments ?
-    func = build_function(
-        getattr(member, "__name__", None) or localname, doc=member.__doc__
-    )
-    node.add_local_node(func, localname)
-    _add_dunder_class(func, member)
+    name = getattr(member, "__name__", "<no-name>")
+    func = build_function(name, node, doc=member.__doc__)
+    _add_dunder_class(func, node, member)
+    return func
 
 
 def _base_class_object_build(
     node: nodes.Module | nodes.ClassDef,
     member: type,
     basenames: list[str],
-    name: str | None = None,
-    localname: str | None = None,
 ) -> nodes.ClassDef:
     """create astroid for a living class object, with a given set of base names
     (e.g. ancestors)
     """
-    class_name = name or getattr(member, "__name__", None) or localname
-    assert isinstance(class_name, str)
-    klass = build_class(
-        class_name,
-        basenames,
-        member.__doc__,
-    )
+    name = getattr(member, "__name__", "<no-name>")
+    doc = member.__doc__ if isinstance(member.__doc__, str) else None
+    klass = build_class(name, node, basenames, doc)
     klass._newstyle = isinstance(member, type)
-    node.add_local_node(klass, localname)
     try:
         # limit the instantiation trick since it's too dangerous
         # (such as infinite test execution...)
@@ -391,10 +390,9 @@ def _base_class_object_build(
 
 def _build_from_function(
     node: nodes.Module | nodes.ClassDef,
-    name: str,
     member: _FunctionTypes,
     module: types.ModuleType,
-) -> None:
+) -> nodes.FunctionDef | nodes.EmptyNode:
     # verify this is not an imported function
     try:
         code = member.__code__  # type: ignore[union-attr]
@@ -404,12 +402,10 @@ def _build_from_function(
         code = None
     filename = getattr(code, "co_filename", None)
     if filename is None:
-        assert isinstance(member, object)
-        object_build_methoddescriptor(node, member, name)
-    elif filename != getattr(module, "__file__", None):
-        attach_dummy_node(node, name, member)
-    else:
-        object_build_function(node, member, name)
+        return object_build_methoddescriptor(node, member)
+    if filename == getattr(module, "__file__", None):
+        return object_build_function(node, member)
+    return build_dummy(member)
 
 
 def _safe_has_attribute(obj, member: str) -> bool:
@@ -432,8 +428,8 @@ class InspectBuilder:
 
     bootstrapped: bool = False
 
-    def __init__(self, manager_instance: AstroidManager | None = None) -> None:
-        self._manager = manager_instance or AstroidManager()
+    def __init__(self, manager_instance: AstroidManager) -> None:
+        self._manager = manager_instance
         self._done: dict[types.ModuleType | type, nodes.Module | nodes.ClassDef] = {}
         self._module: types.ModuleType
 
@@ -476,58 +472,57 @@ class InspectBuilder:
         if obj in self._done:
             return None
         self._done[obj] = node
-        for name in dir(obj):
+        for alias in dir(obj):
             # inspect.ismethod() and inspect.isbuiltin() in PyPy return
             # the opposite of what they do in CPython for __class_getitem__.
-            pypy__class_getitem__ = IS_PYPY and name == "__class_getitem__"
+            pypy__class_getitem__ = IS_PYPY and alias == "__class_getitem__"
             try:
                 with warnings.catch_warnings():
                     warnings.simplefilter("ignore")
-                    member = getattr(obj, name)
+                    member = getattr(obj, alias)
             except AttributeError:
                 # damned ExtensionClass.Base, I know you're there !
-                attach_dummy_node(node, name)
+                attach_dummy_node(node, alias)
                 continue
             if inspect.ismethod(member) and not pypy__class_getitem__:
                 member = member.__func__
             if inspect.isfunction(member):
-                _build_from_function(node, name, member, self._module)
+                child = _build_from_function(node, member, self._module)
             elif inspect.isbuiltin(member) or pypy__class_getitem__:
-                if self.imported_member(node, member, name):
+                if self.imported_member(node, member, alias):
                     continue
-                object_build_methoddescriptor(node, member, name)
+                child = object_build_methoddescriptor(node, member)
             elif inspect.isclass(member):
-                if self.imported_member(node, member, name):
+                if self.imported_member(node, member, alias):
                     continue
                 if member in self._done:
-                    class_node = self._done[member]
-                    assert isinstance(class_node, nodes.ClassDef)
-                    if class_node not in node.locals.get(name, ()):
-                        node.add_local_node(class_node, name)
+                    child = self._done[member]
+                    assert isinstance(child, nodes.ClassDef)
                 else:
-                    class_node = object_build_class(node, member, name)
+                    child = object_build_class(node, member)
                     # recursion
-                    self.object_build(class_node, member)
-                if name == "__class__" and class_node.parent is None:
-                    class_node.parent = self._done[self._module]
+                    self.object_build(child, member)
             elif inspect.ismethoddescriptor(member):
-                object_build_methoddescriptor(node, member, name)
+                child: nodes.NodeNG = object_build_methoddescriptor(node, member)
             elif inspect.isdatadescriptor(member):
-                object_build_datadescriptor(node, member, name)
-            elif isinstance(member, _CONSTANTS):
-                attach_const_node(node, name, member)
+                child = object_build_datadescriptor(node, member)
+            elif isinstance(member, tuple(node_classes.CONST_CLS)):
+                if alias in node.special_attributes:
+                    continue
+                child = nodes.const_factory(member)
             elif inspect.isroutine(member):
                 # This should be called for Jython, where some builtin
                 # methods aren't caught by isbuiltin branch.
-                _build_from_function(node, name, member, self._module)
+                child = _build_from_function(node, member, self._module)
             elif _safe_has_attribute(member, "__all__"):
-                module = build_module(name)
-                _attach_local_node(node, module, name)
+                child: nodes.NodeNG = build_module(alias)
                 # recursion
-                self.object_build(module, member)
+                self.object_build(child, member)
             else:
                 # create an empty node so that the name is actually defined
-                attach_dummy_node(node, name, member)
+                child: nodes.NodeNG = build_dummy(member)
+            if child not in node.locals.get(alias, ()):
+                node.add_local_node(child, alias)
         return None
 
     def imported_member(self, node, member, name: str) -> bool:
@@ -604,19 +599,19 @@ def _astroid_bootstrapping() -> None:
     """astroid bootstrapping the builtins module"""
     # this boot strapping is necessary since we need the Const nodes to
     # inspect_build builtins, and then we can proxy Const
-    builder = InspectBuilder()
+    # pylint: disable-next=import-outside-toplevel
+    from astroid.manager import AstroidManager
+
+    builder = InspectBuilder(AstroidManager())
     astroid_builtin = builder.inspect_build(builtins)
 
     for cls, node_cls in node_classes.CONST_CLS.items():
         if cls is TYPE_NONE:
-            proxy = build_class("NoneType")
-            proxy.parent = astroid_builtin
+            proxy = build_class("NoneType", astroid_builtin)
         elif cls is TYPE_NOTIMPLEMENTED:
-            proxy = build_class("NotImplementedType")
-            proxy.parent = astroid_builtin
+            proxy = build_class("NotImplementedType", astroid_builtin)
         elif cls is TYPE_ELLIPSIS:
-            proxy = build_class("Ellipsis")
-            proxy.parent = astroid_builtin
+            proxy = build_class("Ellipsis", astroid_builtin)
         else:
             proxy = astroid_builtin.getattr(cls.__name__)[0]
             assert isinstance(proxy, nodes.ClassDef)
@@ -634,9 +629,9 @@ def _astroid_bootstrapping() -> None:
         col_offset=0,
         end_lineno=0,
         end_col_offset=0,
-        parent=nodes.Unknown(),
+        parent=astroid_builtin,
     )
-    _GeneratorType.parent = astroid_builtin
+    astroid_builtin.set_local(_GeneratorType.name, _GeneratorType)
     generator_doc_node = (
         nodes.Const(value=types.GeneratorType.__doc__)
         if types.GeneratorType.__doc__
@@ -658,9 +653,9 @@ def _astroid_bootstrapping() -> None:
             col_offset=0,
             end_lineno=0,
             end_col_offset=0,
-            parent=nodes.Unknown(),
+            parent=astroid_builtin,
         )
-        _AsyncGeneratorType.parent = astroid_builtin
+        astroid_builtin.set_local(_AsyncGeneratorType.name, _AsyncGeneratorType)
         async_generator_doc_node = (
             nodes.Const(value=types.AsyncGeneratorType.__doc__)
             if types.AsyncGeneratorType.__doc__
@@ -682,9 +677,8 @@ def _astroid_bootstrapping() -> None:
             col_offset=0,
             end_lineno=0,
             end_col_offset=0,
-            parent=nodes.Unknown(),
+            parent=astroid_builtin,
         )
-        _UnionTypeType.parent = astroid_builtin
         union_type_doc_node = (
             nodes.Const(value=types.UnionType.__doc__)
             if types.UnionType.__doc__
@@ -719,14 +713,14 @@ def _astroid_bootstrapping() -> None:
                 col_offset=0,
                 end_lineno=0,
                 end_col_offset=0,
-                parent=nodes.Unknown(),
+                parent=astroid_builtin,
             )
-            klass.parent = astroid_builtin
+            doc = _type.__doc__ if isinstance(_type.__doc__, str) else None
             klass.postinit(
                 bases=[],
                 body=[],
                 decorators=None,
-                doc_node=nodes.Const(value=_type.__doc__) if _type.__doc__ else None,
+                doc_node=nodes.Const(doc) if doc else None,
             )
             builder.object_build(klass, _type)
             astroid_builtin[_type.__name__] = klass
